// Core
export {
  _, bitset, Predicate, Box, box,
  clock,
  TaskToken, DirtyCheckToken, UnmountToken, TASK_TOKEN, DIRTY_CHECK_TOKEN, UNMOUNT_TOKEN, EMPTY_OBJECT, EMPTY_ARRAY,
  getFunctionName, addErrorHandler, catchError,
  NOOP, NOOP_FALSE, NOOP_TRUE,
  append, unorderedArrayDeleteByIndex, unorderedArrayDelete, orderedArrayFindIndexForInsert,
  RepeatableTaskList, runRepeatableTasks,
  strictEqual, shallowEqual, shallowEqualArray,
  lazy, memo, memoObject, memoArray,
  Observable, ObservableValue, observable, apply, assign, mut, signal, emit, computed, selector, watch,
} from "./core";

// DOM
export {
  PASSIVE_EVENTS, KEYBOARD_EVENT_KEY, MOUSE_EVENT_BUTTONS, TOUCH_EVENTS, POINTER_EVENTS, INPUT_DEVICE_CAPABILITIES,
  IOS_GESTURE_EVENT,
} from "./dom/feature_detection";
export {
  objectHasOwnProperty,
  nodeInsertBefore, nodeRemoveChild, nodeReplaceChild, nodeCloneNode, elementRemoveAttribute, elementSetAttribute,
  elementSetAttributeNS,
} from "./dom/shortcuts";
export {
  SVG_NAMESPACE, XLINK_NAMESPACE, XML_NAMESPACE,
} from "./dom/namespaces";
export { CSSStyleProps } from "./dom/style";
export {
  HTMLAnchorElementAttrs, HTMLElementAttrs, HTMLAreaElementAttrs, HTMLAudioElementAttrs,
  HTMLBaseElementAttrs, HTMLBodyElementAttrs, HTMLBRElementAttrs, HTMLButtonElementAttrs,
  HTMLCanvasElementAttrs, HTMLQuoteElementAttrs, HTMLTableCaptionElementAttrs, HTMLTableColElementAttrs,
  HTMLModElementAttrs, HTMLDivElementAttrs, HTMLDListElementAttrs, HTMLFieldSetElementAttrs, HTMLFormElementAttrs,
  HTMLHeadElementAttrs, HTMLHeadingElementAttrs, HTMLHRElementAttrs, HTMLHtmlElementAttrs, HTMLIFrameElementAttrs,
  HTMLImageElementAttrs, HTMLInputElementAttrs, HTMLLabelElementAttrs, HTMLLegendElementAttrs, HTMLLIElementAttrs,
  HTMLLinkElementAttrs, HTMLMapElementAttrs, HTMLMenuElementAttrs, HTMLMetaElementAttrs, HTMLMeterElementAttrs,
  HTMLOListElementAttrs, HTMLOptGroupElementAttrs, HTMLOptionElementAttrs, HTMLParagraphElementAttrs,
  HTMLPictureElementAttrs, HTMLPreElementAttrs, HTMLProgressElementAttrs, HTMLScriptElementAttrs,
  HTMLSelectElementAttrs, HTMLSourceElementAttrs, HTMLSpanElementAttrs, HTMLStyleElementAttrs,
  HTMLTableDataCellElementAttrs, HTMLTableElementAttrs, HTMLTableHeaderCellElementAttrs, HTMLTableRowElementAttrs,
  HTMLTableSectionElementAttrs, HTMLTemplateElementAttrs, HTMLTextAreaElementAttrs, HTMLTitleElementAttrs,
  HTMLTrackElementAttrs, HTMLUListElementAttrs, HTMLUnknownElementAttrs, HTMLVideoElementAttrs, HTMLMediaElementAttrs,
} from "./dom/html";
export {
  SVGCircleElementAttrs, SVGClipPathElementAttrs, SVGDefsElementAttrs, SVGDescElementAttrs, SVGEllipseElementAttrs,
  SVGFEBlendElementAttrs, SVGFEColorMatrixElementAttrs, SVGFEComponentTransferElementAttrs,
  SVGFECompositeElementAttrs, SVGFEConvolveMatrixElementAttrs, SVGFEDiffuseLightingElementAttrs,
  SVGFEDisplacementMapElementAttrs, SVGFEDistantLightElementAttrs, SVGFEFloodElementAttrs, SVGFEFuncAElementAttrs,
  SVGFEFuncBElementAttrs, SVGFEFuncGElementAttrs, SVGFEFuncRElementAttrs, SVGFEGaussianBlurElementAttrs,
  SVGFEImageElementAttrs, SVGFEMergeElementAttrs, SVGFEMergeNodeElementAttrs, SVGFEMorphologyElementAttrs,
  SVGFEOffsetElementAttrs, SVGFEPointLightElementAttrs, SVGFESpecularLightingElementAttrs, SVGFESpotLightElementAttrs,
  SVGFETileElementAttrs, SVGFETurbulenceElementAttrs, SVGFilterElementAttrs, SVGForeignObjectElementAttrs,
  SVGGElementAttrs, SVGImageElementAttrs, SVGLinearGradientElementAttrs, SVGLineElementAttrs, SVGMarkerElementAttrs,
  SVGMaskElementAttrs, SVGMetadataElementAttrs, SVGPathElementAttrs, SVGPatternElementAttrs, SVGPolygonElementAttrs,
  SVGPolylineElementAttrs, SVGRadialGradientElementAttrs, SVGRectElementAttrs, SVGStopElementAttrs,
  SVGSVGElementAttrs, SVGSwitchElementAttrs, SVGSymbolElementAttrs, SVGTextElementAttrs, SVGTextPathElementAttrs,
  SVGTSpanElementAttrs, SVGViewElementAttrs, SVGUseElementAttrs, SVGElementAttrs,
} from "./dom/svg";
export {
  KeyCode, KeyLocation, KeyName, MouseButtons,
  getEventCharCode, getEventKey, getMouseButtons,
} from "./dom/input";

// Virtual DOM
export { NodeFlags } from "./vdom/node_flags";
export { ContextDescriptor, ContextState, getContext, setContext, pushContext, contextValue } from "./vdom/context";
export {
  AttributeDirective,
  PROPERTY, UNSAFE_HTML, EVENT, AUTOFOCUS,
} from "./vdom/attribute_directive";
export { ElementProtoDescriptor } from "./vdom/element_proto";
export { ComponentState, ComponentDescriptor, Component } from "./vdom/component";
export {
  Key, OpType, DOMElementOp, EventsOp, ContextOp, SetContextStateOp, TrackByKeyOp, ComponentOp, OpNode, Op, OpArray,
  createOpType, createValueOp, createContainerOp, createDOMElementOp,
  Events, TrackByKey, SetContextState, key,
} from "./vdom/operations";
export { OpState, createStateNode } from "./vdom/state";
export {
  elementFactory, htmlElementFactory, svgElementFactory, elementProto, component, statelessComponent,
} from "./vdom/factories";
export { dirtyCheck } from "./vdom/root";
export { useUnmount, useEffect, useMutationEffect, useLayoutEffect, useReducer } from "./vdom/hooks";
export { VisitNodesDirective, visitNodes, getDOMNode } from "./vdom/reconciler";
export { findDOMNode, containsDOMElement, hasDOMElementChild, reducer } from "./vdom/utils";
export { Ref } from "./vdom/ref";

// Scheduler
export {
  withSchedulerTick, addNextFrameMiddleware, withNextFrame, requestDirtyCheck, render, invalidate, requestNextFrame,
  scheduleMicrotask, scheduleMutationEffect, scheduleLayoutEffect, debounceMutationEffect,
  frameStartTime,
} from "./scheduler";

// Events
export {
  getEventTarget,
  EVENT_CAPTURE_PASSIVE_OPTIONS, EVENT_CAPTURE_ACTIVE_OPTIONS, EVENT_PASSIVE_OPTIONS, EVENT_ACTIVE_OPTIONS,
} from "./events/utils";
export { EventHandlerFlags, EventHandlerNode, EventHandler } from "./events/event_handler";
export { DispatchEvent, DispatchTarget, dispatchEvent } from "./events/dispatch";
export {
  NativeEventSource, createNativeEventSource, addNativeEventMiddleware, nativeEventHandlerFactory,

  ABORT_EVENT, ACTIVATE_EVENT, ARIA_REQUEST_EVENT, BEFORE_ACTIVATE_EVENT, BEFORE_COPY_EVENT, BEFORE_CUT_EVENT,
  BEFORE_DEACTIVATE_EVENT, BEFORE_PASTE_EVENT, BLUR_EVENT, CAN_PLAY_EVENT, CAN_PLAYTHROUGH_EVENT, CHANGE_EVENT,
  CLICK_EVENT, CONTEXT_MENU_EVENT, COPY_EVENT, CUE_CHANGE_EVENT, CUT_EVENT, DOUBLE_CLICK_EVENT, DEACTIVATE_EVENT,
  DRAG_EVENT, DRAG_END_EVENT, DRAG_ENTER_EVENT, DRAG_LEAVE_EVENT, DRAG_OVER_EVENT, DRAG_START_EVENT, DROP_EVENT,
  DURATION_CHANGE_EVENT, EMPTIED_EVENT, ENCRYPTED_EVENT, ENDED_EVENT, ERROR_EVENT, FOCUS_EVENT,
  GOT_POINTER_CAPTURE_EVENT, BEFORE_INPUT_EVENT, INPUT_EVENT, INVALID_EVENT, KEY_DOWN_EVENT, KEY_PRESS_EVENT,
  KEY_UP_EVENT, LOAD_EVENT, LOADED_DATA_EVENT, LOADED_METADATA_EVENT, LOAD_START_EVENT, LOST_POINTER_CAPTURE_EVENT,
  MOUSE_DOWN_EVENT, MOUSE_OUT_EVENT, MOUSE_OVER_EVENT, MOUSE_UP_EVENT, PASTE_EVENT, PAUSE_EVENT,
  PLAY_EVENT, PLAYING_EVENT, POINTER_CANCEL_EVENT, POINTER_DOWN_EVENT, POINTER_OUT_EVENT,
  POINTER_OVER_EVENT, POINTER_UP_EVENT, PROGRESS_EVENT, RATE_CHANGE_EVENT, RESET_EVENT, SCROLL_EVENT, SEEKED_EVENT,
  SEEKING_EVENT, SELECT_EVENT, SELECT_START_EVENT, STALLED_EVENT, SUBMIT_EVENT, SUSPEND_EVENT, TIME_UPDATE_EVENT,
  TOUCH_CANCEL_EVENT, TOUCH_END_EVENT, TOUCH_START_EVENT, TRANSITION_CANCEL_EVENT,
  TRANSITION_END_EVENT, TRANSITION_RUN_EVENT, TRANSITION_START_EVENT, UNLOAD_EVENT, VOLUME_CHANGE_EVENT, WAITING_EVENT,
  WHEEL_EVENT, ACTIVE_TOUCH_END_EVENT, ACTIVE_TOUCH_START_EVENT, ACTIVE_WHEEL_EVENT,

  onAbort, onActivate, onAriaRequest, onBeforeActivate, onBeforeCopy, onBeforeCut, onBeforeDeactivate, onBeforePaste,
  onBlur, onCanPlay, onCanPlaythrough, onChange, onClick, onContextMenu, onCopy, onCueChange, onCut, onDoubleClick,
  onDeactivate, onDrag, onDragEnd, onDragEnter, onDragLeave, onDragOver, onDragStart, onDrop, onDurationChange,
  onEmptied, onEncrypted, onEnded, onError, onFocus, onGotPointerCapture, onBeforeInput, onInput, onInvalid, onKeyDown,
  onKeyPress, onKeyUp, onLoad, onLoadedData, onLoadedMetadata, onLoadStart, onLostPointerCapture, onMouseDown,
  onMouseOut, onMouseOver, onMouseUp, onPaste, onPause, onPlay, onPlaying, onPointerCancel, onPointerDown,
  onPointerOut, onPointerOver, onPointerUp, onProgress, onRateChange, onReset, onScroll, onSeeked,
  onSeeking, onSelect, onSelectStart, onStalled, onSubmit, onSuspend, onTimeUpdate, onTouchCancel, onTouchEnd,
  onTouchStart, onTransitionCancel, onTransitionEnd, onTransitionRun, onTransitionStart, onUnload,
  onVolumeChange, onWaiting, onWheel, onActiveTouchEnd, onActiveTouchStart, onActiveWheel,
} from "./events/native_events";
